@@ -23,20 +23,20 @@ import datetime
 import logging
 import os
 import shutil
-import time
 from glob import glob
 
 import dateutil.parser
 import dateutil.tz
 
 from barman import output, xlog
-from barman.backup_executor import RsyncBackupExecutor, SshCommandException
-from barman.command_wrappers import DataTransferFailure
-from barman.compression import CompressionIncompatibility, CompressionManager
+from barman.backup_executor import PostgresBackupExecutor, RsyncBackupExecutor
+from barman.compression import CompressionManager
 from barman.config import BackupOptions
-from barman.hooks import (AbortedRetryHookScript, HookScriptRunner,
-                          RetryHookScriptRunner)
-from barman.infofile import BackupInfo, UnknownBackupIdException, WalFileInfo
+from barman.exceptions import (AbortedRetryHookScript,
+                               CompressionIncompatibility, SshCommandException,
+                               UnknownBackupIdException)
+from barman.hooks import HookScriptRunner, RetryHookScriptRunner
+from barman.infofile import BackupInfo, WalFileInfo
 from barman.recovery_executor import RecoveryExecutor
 from barman.remote_status import RemoteStatusMixin
 from barman.utils import fsync_dir, human_readable_timedelta, pretty_size
@@ -44,21 +44,6 @@ from barman.utils import fsync_dir, human_readable_timedelta, pretty_size
 _logger = logging.getLogger(__name__)
 
 
-class DuplicateWalFile(Exception):
-    """
-    A duplicate WAL file has been found
-    """
-    pass
-
-
-class MatchingDuplicateWalFile(DuplicateWalFile):
-    """
-    A duplicate WAL file has been found, but it's identical to the one we
-    already have.
-    """
-    pass
-
-
 class BackupManager(RemoteStatusMixin):
     """Manager of the backup archive for a server"""
 
@@ -69,18 +54,29 @@ class BackupManager(RemoteStatusMixin):
         Constructor
         """
         super(BackupManager, self).__init__()
-        self.name = "default"
         self.server = server
         self.config = server.config
         self._backup_cache = None
         self.compression_manager = CompressionManager(self.config, server.path)
         self.executor = None
         try:
-            self.executor = RsyncBackupExecutor(self)
+            if self.config.backup_method == "postgres":
+                self.executor = PostgresBackupExecutor(self)
+            else:
+                self.executor = RsyncBackupExecutor(self)
         except SshCommandException as e:
             self.config.disabled = True
             self.config.msg_list.append(str(e).strip())
 
+    @property
+    def mode(self):
+        """
+        Property defining the BackupInfo mode content
+        """
+        if self.executor:
+            return self.executor.mode
+        return None
+
     def get_available_backups(self, status_filter=DEFAULT_STATUS_FILTER):
         """
         Get a list of available backups
@@ -287,8 +283,26 @@ class BackupManager(RemoteStatusMixin):
                 remove_until = next_backup
             elif BackupOptions.CONCURRENT_BACKUP in self.config.backup_options:
                 remove_until = backup
+
+            timelines_to_protect = set()
+            # If remove_until is not set there are no backup left
+            if remove_until:
+                # Retrieve the list of extra timelines that contains at least
+                # a backup. On such timelines we don't want to delete any WAL
+                for value in self.get_available_backups(
+                        BackupInfo.STATUS_ARCHIVING).values():
+                    # Ignore the backup that is being deleted
+                    if value == backup:
+                        continue
+                    timelines_to_protect.add(value.timeline)
+                # Remove the timeline of `remove_until` from the list.
+                # We have enough information to safely delete unused WAL files
+                # on it.
+                timelines_to_protect -= set([remove_until.timeline])
+
             output.info("Delete associated WAL segments:")
-            for name in self.remove_wal_before_backup(remove_until):
+            for name in self.remove_wal_before_backup(remove_until,
+                                                      timelines_to_protect):
                 output.info("\t%s", name)
         # As last action, remove the backup directory,
         # ending the delete operation
@@ -303,43 +317,6 @@ class BackupManager(RemoteStatusMixin):
         self.backup_cache_remove(backup)
         output.info("Done")
 
-    def retry_backup_copy(self, target_function, *args, **kwargs):
-        """
-        Execute the target backup copy function, retrying the configured
-        number of times
-
-        :param target_function: the base backup target function
-        :param args: args for the target function
-        :param kwargs: kwargs of the target function
-        :return: the result of the target function
-        """
-        attempts = 0
-        while True:
-            try:
-                # if is not the first attempt, output the retry number
-                if attempts >= 1:
-                    output.warning("Copy of base backup: retry #%s", attempts)
-                # execute the target function for backup copy
-                return target_function(*args, **kwargs)
-            # catch rsync errors
-            except DataTransferFailure as e:
-                # exit condition: if retry number is lower than configured
-                # retry limit, try again; otherwise exit.
-                if attempts < self.config.basebackup_retry_times:
-                    # Log the exception, for debugging purpose
-                    _logger.exception("Failure in base backup copy: %s", e)
-                    output.warning(
-                        "Copy of base backup failed, waiting for next "
-                        "attempt in %s seconds",
-                        self.config.basebackup_retry_sleep)
-                    # sleep for configured time. then try again
-                    time.sleep(self.config.basebackup_retry_sleep)
-                    attempts += 1
-                else:
-                    # if the max number of attempts is reached and
-                    # there is still an error, exit re-raising the exception.
-                    raise
-
     def backup(self):
         """
         Performs a backup for the server
@@ -355,7 +332,8 @@ class BackupManager(RemoteStatusMixin):
             backup_info.save()
             self.backup_cache_add(backup_info)
             output.info(
-                "Starting backup for server %s in %s",
+                "Starting backup using %s method for server %s in %s",
+                self.mode,
                 self.config.name,
                 backup_info.get_basebackup_directory())
 
@@ -720,14 +698,19 @@ class BackupManager(RemoteStatusMixin):
                     '(history: %s, backup_labels: %s, wal_file: %s)',
                     self.config.name, history_count, label_count, wal_count)
 
-    def remove_wal_before_backup(self, backup_info):
+    def remove_wal_before_backup(self, backup_info, timelines_to_protect=None):
         """
         Remove WAL files which have been archived before the start of
         the provided backup.
 
         If no backup_info is provided delete all available WAL files
 
+        If timelines_to_protect list is passed, never remove a wal in one of
+        these timelines.
+
         :param BackupInfo|None backup_info: the backup information structure
+        :param set timelines_to_protect: optional list of timelines
+            to protect
         :return list: a list of removed WAL files
         """
         removed = []
@@ -743,13 +726,25 @@ class BackupManager(RemoteStatusMixin):
                             "to solve this issue",
                             wal_info.name, self.config.name)
                         continue
-                    # Keeps the WAL segment if it is a history file or later
+
+                    # Keeps the WAL segment if it is a history file
+                    keep = xlog.is_history_file(wal_info.name)
+
+                    # Keeps the WAL segment if its timeline is in
+                    # `timelines_to_protect`
+                    if timelines_to_protect:
+                        tli, _, _ = xlog.decode_segment_name(wal_info.name)
+                        keep |= tli in timelines_to_protect
+
+                    # Keeps the WAL segment if it is a newer
                     # than the given backup (the first available)
-                    if (xlog.is_history_file(wal_info.name) or
-                            (backup_info and
-                                wal_info.name >= backup_info.begin_wal)):
+                    if backup_info:
+                        keep |= wal_info.name >= backup_info.begin_wal
+
+                    # If the file has to be kept write it in the new xlogdb
+                    # otherwise delete it  and record it in the removed list
+                    if keep:
                         fxlogdb_new.write(wal_info.to_xlogdb_line())
-                        continue
                     else:
                         self.delete_wal(wal_info)
                         removed.append(wal_info.name)
