@@ -30,10 +30,10 @@ from argh import ArghParser, arg, expects_obj, named
 import barman.config
 import barman.diagnose
 from barman import output
+from barman.exceptions import BadXlogSegmentName
 from barman.infofile import BackupInfo
 from barman.server import Server
 from barman.utils import configure_logging, drop_privileges, parse_log_level
-from barman.xlog import BadXlogSegmentName
 
 _logger = logging.getLogger(__name__)
 
@@ -423,14 +423,22 @@ def show_server(args):
     """
     Show all configuration parameters for the specified servers
     """
-    servers = get_server_list(args, skip_inactive=True)
+    servers = get_server_list(args)
     for name in sorted(servers):
         server = servers[name]
 
         # Skip the server (apply general rule)
-        if not manage_server_command(server, name):
+        if not manage_server_command(
+                server, name, skip_inactive=False,
+                skip_disabled=False, disabled_is_error=False):
             continue
 
+        # If the server has been manually disabled
+        if not server.config.active:
+            name += " (inactive)"
+        # If server has configuration errors
+        elif server.config.disabled:
+            name += " (WARNING: disabled)"
         output.init('show_server', name)
         with closing(server):
             server.show()
@@ -660,6 +668,10 @@ def archive_wal(args):
 @arg('--reset', help='reset the status of receive-wal removing '
                      'any status files',
      action='store_true')
+@arg('--create-slot', help='create the replication slot, if it does not exist',
+     action='store_true')
+@arg('--drop-slot', help='drop the replication slot, if it exists',
+     action='store_true')
 @arg('server_name',
      completer=server_completer,
      help='specifies the server name for the command')
@@ -677,6 +689,12 @@ def receive_wal(args):
     # termination signal, otherwise attempt to start it
     elif args.stop:
         server.kill('receive-wal')
+    elif args.create_slot:
+        with closing(server):
+            server.create_physical_repslot()
+    elif args.drop_slot:
+        with closing(server):
+            server.drop_repslot()
     else:
         with closing(server):
             server.receive_wal(reset=args.reset)
@@ -963,9 +981,10 @@ def main():
     """
     The main method of Barman
     """
-    p = ArghParser()
+    p = ArghParser(epilog='Barman by 2ndQuadrant (www.2ndQuadrant.com)')
     p.add_argument('-v', '--version', action='version',
-                   version=barman.__version__)
+                   version='%s\n\nBarman by 2ndQuadrant (www.2ndQuadrant.com)'
+                           % barman.__version__)
     p.add_argument('-c', '--config',
                    help='uses a configuration file '
                         '(defaults: %s)'
@@ -1003,7 +1022,7 @@ def main():
         p.dispatch(pre_call=global_config)
     except KeyboardInterrupt:
         msg = "Process interrupted by user (KeyboardInterrupt)"
-        output.exception(msg)
+        output.error(msg)
     except Exception as e:
         msg = "%s\nSee log file for more details." % e
         output.exception(msg)
