@@ -96,7 +96,7 @@ class CsvOption(set):
         Parses a list of values and correctly assign the set of values
         (removing duplication) and checking for conflicts.
         """
-        if value == '':
+        if not value:
             return
         values_list = value.split(',')
         for val in sorted(values_list):
@@ -155,6 +155,8 @@ class BackupOptions(CsvOption):
         Validates backup_option values: currently it makes sure
         that either exclusive_backup or concurrent_backup are set.
         """
+        if len(self) == 0:
+            return
         if self.CONCURRENT_BACKUP not in self \
                 and self.EXCLUSIVE_BACKUP not in self:
             raise ValueError("Invalid configuration value for "
@@ -266,6 +268,7 @@ class ServerConfig(object):
     KEYS = [
         'active',
         'archiver',
+        'archiver_batch_size',
         'backup_directory',
         'backup_method',
         'backup_options',
@@ -273,6 +276,7 @@ class ServerConfig(object):
         'basebackup_retry_sleep',
         'basebackup_retry_times',
         'basebackups_directory',
+        'check_timeout',
         'compression',
         'conninfo',
         'custom_compression_filter',
@@ -298,9 +302,12 @@ class ServerConfig(object):
         'retention_policy',
         'retention_policy_mode',
         'reuse_backup',
+        'slot_name',
         'ssh_command',
         'streaming_archiver',
+        'streaming_archiver_batch_size',
         'streaming_archiver_name',
+        'streaming_backup_name',
         'streaming_conninfo',
         'streaming_wals_directory',
         'tablespace_bandwidth_limit',
@@ -310,11 +317,13 @@ class ServerConfig(object):
 
     BARMAN_KEYS = [
         'archiver',
+        'archiver_batch_size',
         'backup_method',
         'backup_options',
         'bandwidth_limit',
         'basebackup_retry_sleep',
         'basebackup_retry_times',
+        'check_timeout',
         'compression',
         'configuration_files_directory',
         'custom_compression_filter',
@@ -332,20 +341,24 @@ class ServerConfig(object):
         'retention_policy',
         'retention_policy_mode',
         'reuse_backup',
+        'slot_name',
         'streaming_archiver',
+        'streaming_archiver_batch_size',
         'tablespace_bandwidth_limit',
         'wal_retention_policy'
     ]
 
     DEFAULTS = {
         'active': 'true',
-        'archiver': 'on',
+        'archiver': 'off',
+        'archiver_batch_size': '0',
         'backup_directory': '%(barman_home)s/%(name)s',
         'backup_method': 'rsync',
-        'backup_options': "%s" % BackupOptions.EXCLUSIVE_BACKUP,
+        'backup_options': '',
         'basebackup_retry_sleep': '30',
         'basebackup_retry_times': '0',
         'basebackups_directory': '%(backup_directory)s/base',
+        'check_timeout': '30',
         'disabled': 'false',
         'errors_directory': '%(backup_directory)s/errors',
         'immediate_checkpoint': 'false',
@@ -355,7 +368,9 @@ class ServerConfig(object):
         'recovery_options': '',
         'retention_policy_mode': 'auto',
         'streaming_archiver': 'off',
+        'streaming_archiver_batch_size': '0',
         'streaming_archiver_name': 'barman_receive_wal',
+        'streaming_backup_name': 'barman_streaming_backup',
         'streaming_conninfo': '%(conninfo)s',
         'streaming_wals_directory': '%(backup_directory)s/streaming',
         'wal_retention_policy': 'main',
@@ -369,10 +384,12 @@ class ServerConfig(object):
     PARSERS = {
         'active': parse_boolean,
         'archiver': parse_boolean,
+        'archiver_batch_size': int,
         'backup_method': parse_backup_method,
         'backup_options': BackupOptions,
         'basebackup_retry_sleep': int,
         'basebackup_retry_times': int,
+        'check_timeout': int,
         'disabled': parse_boolean,
         'immediate_checkpoint': parse_boolean,
         'last_backup_maximum_age': parse_time_interval,
@@ -380,6 +397,7 @@ class ServerConfig(object):
         'recovery_options': RecoveryOptions,
         'reuse_backup': parse_reuse_backup,
         'streaming_archiver': parse_boolean,
+        'streaming_archiver_batch_size': int,
     }
 
     def invoke_parser(self, key, source, value, new_value):
@@ -417,10 +435,9 @@ class ServerConfig(object):
                 else:
                     value = parser(new_value)
             except Exception as e:
-                output.warning("Invalid configuration value '%s' for key %s"
-                               " in %s: %s",
-                               value, key, source, e)
-                _logger.exception(e)
+                output.warning("Ignoring invalid configuration value '%s' "
+                               "for key %s in %s: %s",
+                               new_value, key, source, e)
         else:
             value = new_value
         return value
@@ -472,6 +489,27 @@ class ServerConfig(object):
         del json_dict['config']
         return json_dict
 
+    def get_bwlimit(self, tablespace=None):
+        """
+        Return the configured bandwidth limit for the provided tablespace
+
+        If tablespace is None, it returns the global bandwidth limit
+
+        :param barman.infofile.Tablespace tablespace: the tablespace to copy
+        :rtype: str
+        """
+        # Default to global bandwidth limit
+        bwlimit = self.bandwidth_limit
+
+        if tablespace:
+            # A tablespace can be copied using a per-tablespace bwlimit
+            tablespaces_bw_limit = self.tablespace_bandwidth_limit
+            if (tablespaces_bw_limit and
+                    tablespace.name in tablespaces_bw_limit):
+                bwlimit = tablespaces_bw_limit[tablespace.name]
+
+        return bwlimit
+
 
 class Config(object):
     """This class represents the barman configuration.
