@@ -27,6 +27,7 @@ import sys
 
 from barman.infofile import BackupInfo
 from barman.utils import pretty_size
+from barman.xlog import diff_lsn
 
 __all__ = [
     'error_occurred', 'debug', 'info', 'warning', 'error', 'exception',
@@ -438,6 +439,19 @@ class ConsoleOutputWriter(object):
                           assertion.key,
                           assertion.value)
 
+        if results['missing_files']:
+            # At least one file is missing, warn the user
+            self.info("")
+            self.info("WARNING")
+            self.info("The following configuration files have not been "
+                      "saved during backup, hence they have not been "
+                      "restored.")
+            self.info("You need to manually restore them "
+                      "in order to start the recovered PostgreSQL instance:")
+            self.info("")
+            for file_name in results['missing_files']:
+                self.info("    %s" % file_name)
+
         if results['delete_barman_xlog']:
             self.info("")
             self.info("After the recovery, please remember to remove the "
@@ -621,6 +635,11 @@ class ConsoleOutputWriter(object):
                     '{percent:.2%}'.format(
                         percent=data['wal_until_next_compression_ratio']))
             self.info("    Last available       : %s", data['wal_last'])
+            if data['children_timelines']:
+                timelines = data['children_timelines']
+                self.info(
+                    "    Reachable timelines  : %s",
+                    ", ".join([str(history.tli) for history in timelines]))
             self.info("")
             self.info("  Catalog information:")
             self.info("    Retention Policy     : %s",
@@ -637,6 +656,12 @@ class ConsoleOutputWriter(object):
                 self.info("  Error:            : %s",
                           data['error'])
 
+        if data['children_timelines']:
+            self.info("")
+            self.info(
+                "WARNING: WAL information is inaccurate due to "
+                "multiple timelines interacting with this backup")
+
     def init_status(self, server_name):
         """
         Init the status command
@@ -748,6 +773,16 @@ class ConsoleOutputWriter(object):
             for standby in standby_info:
                 self.info("")
 
+                # Calculate differences in bytes
+                sent_diff = diff_lsn(standby.sent_location,
+                                     standby.current_location)
+                write_diff = diff_lsn(standby.write_location,
+                                      standby.current_location)
+                flush_diff = diff_lsn(standby.flush_location,
+                                      standby.current_location)
+                replay_diff = diff_lsn(standby.replay_location,
+                                       standby.current_location)
+
                 # Determine the sync stage of the client
                 sync_stage = None
                 if not standby.replay_location:
@@ -757,16 +792,16 @@ class ConsoleOutputWriter(object):
                     client_type = 'standby'
                     max_level = 5
                     # Only standby can replay WAL info
-                    if standby.replay_diff == 0:
+                    if replay_diff == 0:
                         sync_stage = '5/5 Hot standby (max)'
-                    elif standby.flush_diff == 0:
+                    elif flush_diff == 0:
                         sync_stage = '4/5 2-safe'  # remote flush
 
                 # If not yet done, set the sync stage
                 if not sync_stage:
-                    if standby.write_diff == 0:
+                    if write_diff == 0:
                         sync_stage = '3/%s Remote write' % max_level
-                    elif standby.sent_diff == 0:
+                    elif sent_diff == 0:
                         sync_stage = '2/%s WAL Sent (min)' % max_level
                     else:
                         sync_stage = '1/%s 1-safe' % max_level
@@ -801,25 +836,29 @@ class ConsoleOutputWriter(object):
                 self.info("     Current state   : %s (%s)",
                           standby.state,
                           standby.sync_state)
+                if standby.slot_name:
+                    self.info("     Replication slot: %s", standby.slot_name)
                 self.info("     WAL sender PID  : %s", standby.pid)
                 self.info("     Started at      : %s", standby.backend_start)
                 if standby.backend_xmin:
                     self.info("     Standby's xmin  : %s",
                               standby.backend_xmin or '-')
-                self.info("     Sent location   : %s (diff: %s)",
-                          standby.sent_location,
-                          pretty_size(standby.sent_diff))
-                self.info("     Write location  : %s (diff: %s)",
-                          standby.write_location,
-                          pretty_size(standby.write_diff))
+                if standby.sent_location:
+                    self.info("     Sent location   : %s (diff: %s)",
+                              standby.sent_location,
+                              pretty_size(sent_diff))
+                if standby.write_location:
+                    self.info("     Write location  : %s (diff: %s)",
+                              standby.write_location,
+                              pretty_size(write_diff))
                 if standby.flush_location:
                     self.info("     Flush location  : %s (diff: %s)",
                               standby.flush_location,
-                              pretty_size(standby.flush_diff))
+                              pretty_size(flush_diff))
                 if standby.replay_location:
                     self.info("     Replay location : %s (diff: %s)",
                               standby.replay_location,
-                              pretty_size(standby.replay_diff))
+                              pretty_size(replay_diff))
                 n += 1
 
     def init_list_server(self, server_name, minimal=False):
