@@ -24,46 +24,36 @@ import logging
 from abc import ABCMeta
 
 import psycopg2
+from psycopg2.errorcodes import (DUPLICATE_OBJECT, OBJECT_IN_USE,
+                                 UNDEFINED_OBJECT)
 from psycopg2.extensions import STATUS_IN_TRANSACTION
-from psycopg2.extras import RealDictCursor
+from psycopg2.extras import DictCursor, NamedTupleCursor
 
+from barman.exceptions import (ConninfoException, PostgresAppNameError,
+                               PostgresConnectionError,
+                               PostgresDuplicateReplicationSlot,
+                               PostgresException,
+                               PostgresInvalidReplicationSlot,
+                               PostgresIsInRecovery,
+                               PostgresReplicationSlotInUse,
+                               PostgresReplicationSlotsFull,
+                               PostgresSuperuserRequired,
+                               PostgresUnsupportedFeature)
 from barman.infofile import Tablespace
 from barman.remote_status import RemoteStatusMixin
-from barman.utils import with_metaclass
+from barman.utils import simplify_version, with_metaclass
+
+# This is necessary because the CONFIGURATION_LIMIT_EXCEEDED constant
+# has been added in psycopg2 2.5, but Barman supports version 2.4.2+ so
+# in case of import error we declare a constant providing the correct value.
+try:
+    from psycopg2.errorcodes import CONFIGURATION_LIMIT_EXCEEDED
+except ImportError:
+    CONFIGURATION_LIMIT_EXCEEDED = '53400'
+
 
 _logger = logging.getLogger(__name__)
 
-
-class ConninfoException(Exception):
-    """
-    Error for missing or failed parsing of the conninfo parameter (DSN)
-    """
-
-
-class PostgresConnectionError(Exception):
-    """
-    Error connecting to the PostgreSQL server
-    """
-
-
-class PostgresSuperuserRequired(Exception):
-    """
-    Superuser access is required
-    """
-
-
-class PostgresIsInRecovery(Exception):
-    """
-    PostgreSQL is in recovery, so no write operations are allowed
-    """
-
-
-class PostgresUnsupportedFeature(Exception):
-    """
-    Unsupported feature
-    """
-
-
 _live_connections = []
 """
 List of connections to be closed at the interpreter shutdown
@@ -120,7 +110,35 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         :rtype: dict[str,str]
         """
         # TODO: this might be made more robust in the future
-        return dict(x.split('=', 1) for x in dsn.split(' '))
+        return dict(x.split('=', 1) for x in dsn.split())
+
+    @staticmethod
+    def encode_dsn(parameters):
+        """
+        Build a connection string from a dictionary of connection
+        parameters
+
+        :param dict[str,str] parameters: Connection parameters
+        :rtype: str
+        """
+        # TODO: this might be made more robust in the future
+        return ' '.join(
+            ["%s=%s" % (k, v) for k, v in sorted(parameters.items())])
+
+    def get_connection_string(self, application_name=None):
+        """
+        Return the connection string, adding the application_name parameter
+        if requested, unless already defined by user in the connection string
+
+        :param str application_name: the application_name to add
+        :return str: the connection string
+        """
+        conn_string = self.conninfo
+        # check if the application name is already defined by user
+        if application_name and 'application_name' not in self.conn_parameters:
+            # Then add the it to the connection string
+            conn_string += ' application_name=%s' % application_name
+        return conn_string
 
     def connect(self):
         """
@@ -132,8 +150,7 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
             # If psycopg2 fails to connect to the host,
             # raise the appropriate exception
             except psycopg2.DatabaseError as e:
-                raise PostgresConnectionError(
-                    "Cannot connect to postgres: %s" % str(e).strip())
+                raise PostgresConnectionError(str(e).strip())
             # Register the connection to the live connections list
             _live_connections.append(self)
         return self._conn
@@ -165,6 +182,36 @@ class PostgreSQL(with_metaclass(ABCMeta, RemoteStatusMixin)):
         conn = self.connect()
         return conn.server_version
 
+    @property
+    def server_txt_version(self):
+        """
+        Human readable version of PostgreSQL (calculated from server_version)
+
+        :rtype: str|None
+        """
+        try:
+            conn = self.connect()
+            major = int(conn.server_version / 10000)
+            minor = int(conn.server_version / 100 % 100)
+            patch = int(conn.server_version % 100)
+            return "%d.%d.%d" % (major, minor, patch)
+        except PostgresConnectionError as e:
+            _logger.debug("Error retrieving PostgreSQL version: %s",
+                          str(e).strip())
+            return None
+
+    @property
+    def server_major_version(self):
+        """
+        PostgreSQL major version (calculated from server_txt_version)
+
+        :rtype: str|None
+        """
+        result = self.server_txt_version
+        if result is not None:
+            return simplify_version(result)
+        return None
+
 
 class StreamingConnection(PostgreSQL):
     """
@@ -179,16 +226,19 @@ class StreamingConnection(PostgreSQL):
         """
         if config.streaming_conninfo is None:
             raise ConninfoException(
-                'Missing streaming_conninfo parameter in barman configuration '
-                'for server %s' % config.name
-            )
+                "Missing 'streaming_conninfo' parameter for server '%s'"
+                % config.name)
+
         super(StreamingConnection, self).__init__(config,
                                                   config.streaming_conninfo)
         # Make sure we connect using the 'replication' option which
         # triggers streaming replication protocol communication
-        if 'replication' not in self.conn_parameters:
-            self.conn_parameters['replication'] = 'true'
-            self.conninfo += ' replication=true'
+        self.conn_parameters['replication'] = 'true'
+        # Override 'dbname' parameter. This operation is required to mimic
+        # the behaviour of pg_receivexlog and pg_basebackup
+        self.conn_parameters['dbname'] = 'replication'
+        # Rebuild the conninfo string from the modified parameter lists
+        self.conninfo = self.encode_dsn(self.conn_parameters)
 
     def connect(self):
         """
@@ -202,22 +252,6 @@ class StreamingConnection(PostgreSQL):
             self._conn.autocommit = True
         return self._conn
 
-    @property
-    def server_txt_version(self):
-        """
-        Human readable version of PostgreSQL (calculated from server_version)
-        """
-        try:
-            conn = self.connect()
-            major = int(conn.server_version / 10000)
-            minor = int(conn.server_version / 100 % 100)
-            patch = int(conn.server_version % 100)
-            return "%d.%d.%d" % (major, minor, patch)
-        except PostgresConnectionError as e:
-            _logger.debug("Error retrieving PostgreSQL version: %s",
-                          str(e).strip())
-            return None
-
     def fetch_remote_status(self):
         """
         Returns the status of the connection to the PostgreSQL server.
@@ -228,7 +262,8 @@ class StreamingConnection(PostgreSQL):
         :rtype: dict[str, None|str]
         """
         result = dict.fromkeys(
-            ('streaming_supported', 'streaming', 'systemid',
+            ('connection_error', 'streaming_supported',
+                'streaming', 'systemid',
                 'timeline', 'xlogpos'),
             None)
         try:
@@ -258,10 +293,58 @@ class StreamingConnection(PostgreSQL):
             # This is not a streaming connection
             result['streaming'] = False
         except PostgresConnectionError as e:
+            result['connection_error'] = str(e).strip()
             _logger.warn("Error retrieving PostgreSQL status: %s",
                          str(e).strip())
         return result
 
+    def create_physical_repslot(self, slot_name):
+        """
+        Create a physical replication slot using the streaming connection
+        :param str slot_name: Replication slot name
+        """
+        cursor = self._cursor()
+        try:
+            # In the following query, the slot name is directly passed
+            # to the CREATE_REPLICATION_SLOT command, without any
+            # quoting. This is a characteristic of the streaming
+            # connection, otherwise if will fail with a generic
+            # "syntax error"
+            cursor.execute('CREATE_REPLICATION_SLOT %s PHYSICAL' % slot_name)
+        except psycopg2.DatabaseError as exc:
+            if exc.pgcode == DUPLICATE_OBJECT:
+                # A replication slot with the same name exists
+                raise PostgresDuplicateReplicationSlot()
+            elif exc.pgcode == CONFIGURATION_LIMIT_EXCEEDED:
+                # Unable to create a new physical replication slot.
+                # All slots are full.
+                raise PostgresReplicationSlotsFull()
+            else:
+                raise PostgresException(str(exc).strip())
+
+    def drop_repslot(self, slot_name):
+        """
+        Drop a physical replication slot using the streaming connection
+        :param str slot_name: Replication slot name
+        """
+        cursor = self._cursor()
+        try:
+            # In the following query, the slot name is directly passed
+            # to the DROP_REPLICATION_SLOT command, without any
+            # quoting. This is a characteristic of the streaming
+            # connection, otherwise if will fail with a generic
+            # "syntax error"
+            cursor.execute('DROP_REPLICATION_SLOT %s' % slot_name)
+        except psycopg2.DatabaseError as exc:
+            if exc.pgcode == UNDEFINED_OBJECT:
+                # A replication slot with the that name does not exist
+                raise PostgresInvalidReplicationSlot()
+            if exc.pgcode == OBJECT_IN_USE:
+                # The replication slot is still in use
+                raise PostgresReplicationSlotInUse()
+            else:
+                raise PostgresException(str(exc).strip())
+
 
 class PostgreSQLConnection(PostgreSQL):
     """
@@ -282,8 +365,7 @@ class PostgreSQLConnection(PostgreSQL):
         # Check that 'conninfo' option is properly set
         if config.conninfo is None:
             raise ConninfoException(
-                'Missing conninfo parameter in barman configuration '
-                'for server %s' % config.name)
+                "Missing 'conninfo' parameter for server '%s'" % config.name)
         super(PostgreSQLConnection, self).__init__(config, config.conninfo)
         self.configuration_files = None
 
@@ -304,8 +386,7 @@ class PostgreSQLConnection(PostgreSQL):
             # If psycopg2 fails to set the application name,
             # raise the appropriate exception
             except psycopg2.ProgrammingError as e:
-                raise PostgresConnectionError(
-                    "Cannot set the application name: %s" % str(e).strip())
+                raise PostgresAppNameError(str(e).strip())
         return self._conn
 
     @property
@@ -374,23 +455,56 @@ class PostgreSQLConnection(PostgreSQL):
             return None
 
     @property
-    def current_xlog(self):
+    def current_xlog_info(self):
+        """
+        Get detailed information about the current WAL position in PostgreSQL.
+
+        This method returns a dictionary containing the following data:
+
+         * location
+         * file_name
+         * file_offset
+         * timestamp
+
+        When executed on a standby server file_name and file_offset are always
+        None
+
+        :rtype: psycopg2.extras.DictRow
+        """
+        try:
+            cur = self._cursor(cursor_factory=DictCursor)
+            if not self.is_in_recovery:
+                cur.execute(
+                    "SELECT location, "
+                    "(pg_xlogfile_name_offset(location)).*, "
+                    "CURRENT_TIMESTAMP AS timestamp "
+                    "FROM pg_current_xlog_location() AS location")
+                return cur.fetchone()
+            else:
+                cur.execute(
+                    "SELECT location, "
+                    "NULL AS file_name, "
+                    "NULL AS file_offset, "
+                    "CURRENT_TIMESTAMP AS timestamp "
+                    "FROM pg_last_xlog_replay_location() AS location")
+                return cur.fetchone()
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            _logger.debug("Error retrieving current xlog "
+                          "detailed information: %s",
+                          str(e).strip())
+        return None
+
+    @property
+    def current_xlog_file_name(self):
         """
         Get current WAL file from PostgreSQL
 
         :return str: current WAL file in PostgreSQL
         """
-        try:
-            if not self.is_in_recovery:
-                cur = self._cursor()
-                cur.execute(
-                    'SELECT pg_xlogfile_name('
-                    'pg_current_xlog_location())')
-                return cur.fetchone()[0]
-        except (PostgresConnectionError, psycopg2.Error) as e:
-            _logger.debug("Error retrieving current xlog: %s",
-                          str(e).strip())
-            return None
+        current_xlog_info = self.current_xlog_info
+        if current_xlog_info is not None:
+            return current_xlog_info['file_name']
+        return None
 
     @property
     def current_xlog_location(self):
@@ -399,16 +513,10 @@ class PostgreSQLConnection(PostgreSQL):
 
         :return str: current WAL location in PostgreSQL
         """
-        try:
-            if not self.is_in_recovery:
-                cur = self._cursor()
-                cur.execute(
-                    'SELECT pg_current_xlog_location()')
-                return cur.fetchone()[0]
-        except (PostgresConnectionError, psycopg2.Error) as e:
-            _logger.debug("Error retrieving current xlog location: %s",
-                          str(e).strip())
-            return None
+        current_xlog_info = self.current_xlog_info
+        if current_xlog_info is not None:
+            return current_xlog_info['location']
+        return None
 
     @property
     def current_size(self):
@@ -441,7 +549,7 @@ class PostgreSQLConnection(PostgreSQL):
             # pg_stat_archiver is only available from Postgres 9.4+
             if self.server_version < 90400:
                 return None
-            cur = self._cursor(cursor_factory=RealDictCursor)
+            cur = self._cursor(cursor_factory=DictCursor)
             # Select from pg_stat_archiver statistics view,
             # retrieving statistics about WAL archiver process activity,
             # also evaluating if the server is archiving without issues
@@ -488,7 +596,11 @@ class PostgreSQLConnection(PostgreSQL):
             'server_txt_version',
             'is_superuser',
             'current_xlog',
-            'pgespresso_installed']
+            'pgespresso_installed',
+            'replication_slot_support',
+            'replication_slot',
+            'synchronous_standby_names',
+        ]
         # Initialise the result dictionary setting all the values to None
         result = dict.fromkeys(pg_superuser_settings +
                                pg_settings +
@@ -511,10 +623,25 @@ class PostgreSQLConnection(PostgreSQL):
             result['is_superuser'] = self.is_superuser
             result['server_txt_version'] = self.server_txt_version
             result['pgespresso_installed'] = self.has_pgespresso
-            result['current_xlog'] = self.current_xlog
+            result['current_xlog'] = self.current_xlog_file_name
             result['current_size'] = self.current_size
 
             result.update(self.get_configuration_files())
+
+            # Retrieve the replication_slot status
+            result["replication_slot_support"] = False
+            if self.server_version >= 90400:
+                result["replication_slot_support"] = True
+                if self.config.slot_name is not None:
+                    result["replication_slot"] = (
+                        self.get_replication_slot(self.config.slot_name))
+
+            # Retrieve the list of synchronous standby names
+            result["synchronous_standby_names"] = []
+            if self.server_version >= 90100:
+                result["synchronous_standby_names"] = (
+                    self.get_synchronous_standby_names())
+
         except (PostgresConnectionError, psycopg2.Error) as e:
             _logger.warn("Error retrieving PostgreSQL status: %s",
                          str(e).strip())
@@ -633,34 +760,43 @@ class PostgreSQLConnection(PostgreSQL):
                           'command: %s', str(e).strip())
             return None
 
-    def start_exclusive_backup(self, backup_label):
+    def start_exclusive_backup(self, label):
         """
         Calls pg_start_backup() on the PostgreSQL server
 
-        :param str backup_label: label for the backup returned by Postgres
-        :rtype: tuple
+        This method returns a dictionary containing the following data:
+
+         * location
+         * file_name
+         * file_offset
+         * timestamp
+
+        :param str label: descriptive string to identify the backup
+        :rtype: psycopg2.extras.DictRow
         """
         try:
             conn = self.connect()
 
-            # Issue a rollback to release any unneeded lock
+            # Rollback to release the transaction, as the pg_start_backup
+            # invocation can last up to PostgreSQL's checkpoint_timeout
             conn.rollback()
 
-            # Start the exclusive backup
-            cur = conn.cursor()
+            # Start an exclusive backup
+            cur = conn.cursor(cursor_factory=DictCursor)
             if self.server_version < 80400:
                 cur.execute(
-                    "SELECT xlog_loc, "
-                    "(pg_xlogfile_name_offset(xlog_loc)).*, "
-                    "now() FROM pg_start_backup(%s) as xlog_loc",
-                    (backup_label,))
+                    "SELECT location, "
+                    "(pg_xlogfile_name_offset(location)).*, "
+                    "now() AS timestamp "
+                    "FROM pg_start_backup(%s) AS location",
+                    (label,))
             else:
                 cur.execute(
-                    "SELECT xlog_loc, "
-                    "(pg_xlogfile_name_offset(xlog_loc)).*, "
-                    "now() FROM pg_start_backup(%s,%s) as xlog_loc",
-                    (backup_label,
-                     self.config.immediate_checkpoint))
+                    "SELECT location, "
+                    "(pg_xlogfile_name_offset(location)).*, "
+                    "now() AS timestamp "
+                    "FROM pg_start_backup(%s,%s) AS location",
+                    (label, self.config.immediate_checkpoint))
 
             start_row = cur.fetchone()
 
@@ -672,47 +808,152 @@ class PostgreSQLConnection(PostgreSQL):
         except (PostgresConnectionError, psycopg2.Error) as e:
             msg = "pg_start_backup(): %s" % str(e).strip()
             _logger.debug(msg)
-            raise Exception(msg)
+            raise PostgresException(msg)
+
+    def start_concurrent_backup(self, label):
+        """
+        Calls pg_start_backup on the PostgreSQL server using the
+        API introduced with version 9.6
+
+        This method returns a dictionary containing the following data:
+
+         * location
+         * timeline
+         * timestamp
+
+        :param str label: descriptive string to identify the backup
+        :rtype: psycopg2.extras.DictRow
+        """
+        try:
+            conn = self.connect()
+
+            # Rollback to release the transaction, as the pg_start_backup
+            # invocation can last up to PostgreSQL's checkpoint_timeout
+            conn.rollback()
+
+            # Start the backup using the api introduced in postgres 9.6
+            cur = conn.cursor(cursor_factory=DictCursor)
+            cur.execute(
+                "SELECT location, "
+                "(SELECT timeline_id "
+                "FROM pg_control_checkpoint()) AS timeline, "
+                "now() AS timestamp "
+                "FROM pg_start_backup(%s, %s, FALSE) AS location",
+                (label, self.config.immediate_checkpoint))
+            start_row = cur.fetchone()
+
+            # Rollback to release the transaction, as the connection
+            # is to be retained until the end of backup
+            conn.rollback()
+
+            return start_row
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            msg = "pg_start_backup command: %s" % (str(e).strip(),)
+            _logger.debug(msg)
+            raise PostgresException(msg)
 
     def stop_exclusive_backup(self):
         """
         Calls pg_stop_backup() on the PostgreSQL server
 
-        :returns: a tuple with the result of the pg_stop_backup() call or None
-        :rtype: tuple|None
+        This method returns a dictionary containing the following data:
+
+         * location
+         * file_name
+         * file_offset
+         * timestamp
+
+        :rtype: psycopg2.extras.DictRow
         """
         try:
             conn = self.connect()
-            # Issue a rollback to release any unneeded lock
+
+            # Rollback to release the transaction, as the pg_stop_backup
+            # invocation could will wait until the current WAL file is shipped
             conn.rollback()
-            cur = conn.cursor()
+
+            # Stop the backup
+            cur = conn.cursor(cursor_factory=DictCursor)
             cur.execute(
-                'SELECT xlog_loc, (pg_xlogfile_name_offset(xlog_loc)).*, '
-                'now() FROM pg_stop_backup() as xlog_loc')
+                "SELECT location, "
+                "(pg_xlogfile_name_offset(location)).*, "
+                "now() AS timestamp "
+                "FROM pg_stop_backup() AS location")
+
             return cur.fetchone()
         except (PostgresConnectionError, psycopg2.Error) as e:
-            _logger.debug('Error issuing pg_stop_backup() command: %s',
-                          str(e).strip())
-            return None
+            msg = "Error issuing pg_stop_backup command: %s" % str(e).strip()
+            _logger.debug(msg)
+            raise PostgresException(
+                'Cannot terminate exclusive backup. '
+                'You might have to manually execute pg_stop_backup '
+                'on your PostgreSQL server')
 
-    def pgespresso_start_backup(self, backup_label):
+    def stop_concurrent_backup(self):
+        """
+        Calls pg_stop_backup on the PostgreSQL server using the
+        API introduced with version 9.6
+
+        This method returns a dictionary containing the following data:
+
+         * location
+         * timeline
+         * backup_label
+         * timestamp
+
+        :rtype: psycopg2.extras.DictRow
+        """
+        try:
+            conn = self.connect()
+
+            # Rollback to release the transaction, as the pg_stop_backup
+            # invocation could will wait until the current WAL file is shipped
+            conn.rollback()
+
+            # Stop the backup  using the api introduced with version 9.6
+            cur = conn.cursor(cursor_factory=DictCursor)
+            cur.execute(
+                'SELECT end_row.lsn AS location, '
+                '(SELECT CASE WHEN pg_is_in_recovery() '
+                'THEN min_recovery_end_timeline ELSE timeline_id END '
+                'FROM pg_control_checkpoint(), pg_control_recovery()'
+                ') AS timeline, '
+                'end_row.labelfile AS backup_label, '
+                'now() AS timestamp FROM pg_stop_backup(FALSE) AS end_row')
+
+            return cur.fetchone()
+        except (PostgresConnectionError, psycopg2.Error) as e:
+            msg = "Error issuing pg_stop_backup command: %s" % str(e).strip()
+            _logger.debug(msg)
+            raise PostgresException(msg)
+
+    def pgespresso_start_backup(self, label):
         """
         Execute a pgespresso_start_backup
 
-        :param str backup_label: label for the backup
-        :rtype: tuple
+        This method returns a dictionary containing the following data:
+
+         * backup_label
+         * timestamp
+
+        :param str label: descriptive string to identify the backup
+        :rtype: psycopg2.extras.DictRow
         """
         try:
             conn = self.connect()
 
-            # Issue a rollback to release any unneeded lock
+            # Rollback to release the transaction,
+            # as the pgespresso_start_backup invocation can last
+            # up to PostgreSQL's checkpoint_timeout
             conn.rollback()
 
-            # Start the concurrent backup
-            cur = conn.cursor()
+            # Start the concurrent backup using pgespresso
+            cur = conn.cursor(cursor_factory=DictCursor)
             cur.execute(
-                'SELECT pgespresso_start_backup(%s,%s), now()',
-                (backup_label, self.config.immediate_checkpoint))
+                'SELECT pgespresso_start_backup(%s,%s) AS backup_label, '
+                'now() AS timestamp',
+                (label, self.config.immediate_checkpoint))
+
             start_row = cur.fetchone()
 
             # Rollback to release the transaction, as the connection
@@ -723,30 +964,39 @@ class PostgreSQLConnection(PostgreSQL):
         except (PostgresConnectionError, psycopg2.Error) as e:
             msg = "pgespresso_start_backup(): %s" % str(e).strip()
             _logger.debug(msg)
-            raise Exception(msg)
+            raise PostgresException(msg)
 
     def pgespresso_stop_backup(self, backup_label):
         """
         Execute a pgespresso_stop_backup
 
-        :param str backup_label: label of the backup
-        :returns: a string containing the result of the
-            pgespresso_stop_backup call or None
-        :rtype: tuple|None
+        This method returns a dictionary containing the following data:
+
+         * end_wal
+         * timestamp
+
+        :param str backup_label: backup label as returned
+            by pgespress_start_backup
+        :rtype: psycopg2.extras.DictRow
         """
         try:
             conn = self.connect()
             # Issue a rollback to release any unneeded lock
             conn.rollback()
-            cur = conn.cursor()
-            cur.execute("SELECT pgespresso_stop_backup(%s), now()",
+            cur = conn.cursor(cursor_factory=DictCursor)
+            cur.execute("SELECT pgespresso_stop_backup(%s) AS end_wal, "
+                        "now() AS timestamp",
                         (backup_label,))
             return cur.fetchone()
         except (PostgresConnectionError, psycopg2.Error) as e:
-            _logger.debug(
-                "Error issuing pgespresso_stop_backup() command: %s",
+            msg = "Error issuing pgespresso_stop_backup() command: %s" % (
                 str(e).strip())
-            return None
+            _logger.debug(msg)
+            raise PostgresException(
+                '%s\n'
+                'HINT: You might have to manually execute '
+                'pgespresso_abort_backup() on your PostgreSQL '
+                'server' % msg)
 
     def switch_xlog(self):
         """
@@ -817,7 +1067,7 @@ class PostgreSQLConnection(PostgreSQL):
         Returns streaming replication information
         """
         try:
-            cur = self._cursor(cursor_factory=psycopg2.extras.NamedTupleCursor)
+            cur = self._cursor(cursor_factory=NamedTupleCursor)
 
             # Without superuser rights, this function is useless
             # TODO: provide a simplified version for non-superusers
@@ -850,9 +1100,16 @@ class PostgreSQLConnection(PostgreSQL):
             if self.server_version < 90100:
                 raise PostgresUnsupportedFeature('9.1')
 
-            if self.server_version >= 90400:
-                # Current implementation (9.4+)
-                what = "* "
+            from_repslot = ""
+            if self.server_version >= 90500:
+                # Current implementation (9.5+)
+                what = "r.*, rs.slot_name"
+                # Look for replication slot name
+                from_repslot = "LEFT JOIN pg_replication_slots rs " \
+                               "ON (r.pid = rs.active_pid) "
+            elif self.server_version >= 90400:
+                # PostgreSQL 9.4
+                what = "*"
             elif self.server_version >= 90200:
                 # PostgreSQL 9.2/9.3
                 what = "pid," \
@@ -903,21 +1160,17 @@ class PostgreSQLConnection(PostgreSQL):
             # Execute the query
             cur.execute(
                 "SELECT %s, "
-                "CASE WHEN pg_is_in_recovery() THEN NULL ELSE "
-                "pg_xlog_location_diff(sent_location, "
-                "pg_current_xlog_location()) END AS sent_diff, "
-                "CASE WHEN pg_is_in_recovery() THEN NULL ELSE "
-                "pg_xlog_location_diff(write_location, "
-                "pg_current_xlog_location()) END AS write_diff, "
-                "CASE WHEN pg_is_in_recovery() THEN NULL ELSE "
-                "pg_xlog_location_diff(flush_location, "
-                "pg_current_xlog_location()) END AS flush_diff, "
-                "CASE WHEN pg_is_in_recovery() THEN NULL ELSE "
-                "pg_xlog_location_diff(replay_location, "
-                "pg_current_xlog_location()) END AS replay_diff "
-                "FROM pg_stat_replication "
+                "pg_is_in_recovery() AS is_in_recovery,"
+                "CASE WHEN pg_is_in_recovery() "
+                "  THEN NULL "
+                "  ELSE pg_current_xlog_location() "
+                "END AS current_location "
+                "FROM pg_stat_replication r "
                 "%s"
-                "ORDER BY sync_state DESC, sync_priority" % (what, where))
+                "%s"
+                "ORDER BY sync_state DESC, sync_priority" % (what,
+                                                             from_repslot,
+                                                             where))
 
             # Generate a list of standby objects
             return cur.fetchall()
@@ -925,3 +1178,65 @@ class PostgreSQLConnection(PostgreSQL):
             _logger.debug("Error retrieving status of standby servers: %s",
                           str(e).strip())
             return None
+
+    def get_replication_slot(self, slot_name):
+        """
+        Retrieve from the PostgreSQL server a physical replication slot
+        with a specific slot_name.
+
+        This method returns a dictionary containing the following data:
+
+         * slot_name
+         * active
+         * restart_lsn
+
+        :param str slot_name: the replication slot name
+        :rtype: psycopg2.extras.DictRow
+        """
+        if self.server_version < 90400:
+            # Raise exception if replication slot are not supported
+            # by PostgreSQL version
+            raise PostgresUnsupportedFeature('9.4')
+        else:
+            cur = self._cursor(cursor_factory=NamedTupleCursor)
+            try:
+                cur.execute("SELECT slot_name, "
+                            "active, "
+                            "restart_lsn "
+                            "FROM pg_replication_slots "
+                            "WHERE slot_type = 'physical' "
+                            "AND slot_name = '%s'" % slot_name)
+                # Retrieve the replication slot information
+                return cur.fetchone()
+            except (PostgresConnectionError, psycopg2.Error) as e:
+                _logger.debug("Error retrieving replication_slots: %s",
+                              str(e).strip())
+                raise
+
+    def get_synchronous_standby_names(self):
+        """
+        Retrieve the list of named synchronous standby servers from PostgreSQL
+
+        This method returns a list of names
+
+        :return list: synchronous standby names
+        """
+        if self.server_version < 90100:
+            # Raise exception if synchronous replication is not supported
+            raise PostgresUnsupportedFeature('9.1')
+        else:
+            synchronous_standby_names = (
+                self.get_setting('synchronous_standby_names'))
+            # Normalise the list of sync standby names
+            # On PostgreSQL 9.6 it is possible to specify the number of
+            # required synchronous standby using this format:
+            # n (name1, name2, ... nameN).
+            # We only need the name list, so we discard everything else.
+
+            # The name list starts after the first parenthesis or at pos 0
+            names_start = synchronous_standby_names.find('(') + 1
+            names_end = synchronous_standby_names.rfind(')')
+            if names_end < 0:
+                names_end = len(synchronous_standby_names)
+            names_list = synchronous_standby_names[names_start:names_end]
+            return [x.strip() for x in names_list.split(',')]
