@@ -34,20 +34,29 @@ from barman import output, xlog
 from barman.backup import BackupManager
 from barman.command_wrappers import BarmanSubProcess
 from barman.compression import identify_compression
-from barman.infofile import BackupInfo, UnknownBackupIdException, WalFileInfo
-from barman.lockfile import (LockFileBusy, LockFilePermissionDenied,
-                             ServerBackupLock, ServerCronLock,
+from barman.exceptions import (ArchiverFailure, BadXlogSegmentName,
+                               ConninfoException, LockFileBusy,
+                               LockFilePermissionDenied,
+                               PostgresDuplicateReplicationSlot,
+                               PostgresException,
+                               PostgresInvalidReplicationSlot,
+                               PostgresIsInRecovery,
+                               PostgresReplicationSlotInUse,
+                               PostgresReplicationSlotsFull,
+                               PostgresSuperuserRequired,
+                               PostgresUnsupportedFeature, TimeoutError,
+                               UnknownBackupIdException)
+from barman.infofile import BackupInfo, WalFileInfo
+from barman.lockfile import (ServerBackupLock, ServerCronLock,
                              ServerWalArchiveLock, ServerWalReceiveLock,
                              ServerXLOGDBLock)
-from barman.postgres import (ConninfoException, PostgresIsInRecovery,
-                             PostgreSQLConnection, PostgresSuperuserRequired,
-                             PostgresUnsupportedFeature, StreamingConnection)
+from barman.postgres import PostgreSQLConnection, StreamingConnection
 from barman.process import ProcessManager
 from barman.remote_status import RemoteStatusMixin
 from barman.retention_policies import RetentionPolicyFactory
-from barman.utils import human_readable_timedelta, pretty_size
-from barman.wal_archiver import (ArchiverFailure, FileWalArchiver,
-                                 StreamingWalArchiver, WalArchiver)
+from barman.utils import human_readable_timedelta, pretty_size, timeout
+from barman.wal_archiver import (FileWalArchiver, StreamingWalArchiver,
+                                 WalArchiver)
 
 _logger = logging.getLogger(__name__)
 
@@ -164,20 +173,47 @@ class Server(RemoteStatusMixin):
         self.config = config
         self.path = self._build_path(self.config.path_prefix)
         self.process_manager = ProcessManager(self.config)
-        self.backup_manager = BackupManager(self)
+
         self.enforce_retention_policies = False
         self.postgres = None
         self.streaming = None
         self.archivers = []
 
+        # Initialize the backup manager
+        self.backup_manager = BackupManager(self)
+
+        # Initialize the main PostgreSQL connection
         try:
             self.postgres = PostgreSQLConnection(config)
         # If the PostgreSQLConnection creation fails, disable the Server
         except ConninfoException as e:
             self.config.disabled = True
-            self.config.msg_list.append("conninfo: " + str(e).strip())
+            self.config.msg_list.append("PostgreSQL connection: " +
+                                        str(e).strip())
 
-        # Order of items in self.archivers list is important!
+        # ARCHIVER_OFF_BACKCOMPATIBILITY - START OF CODE
+        # IMPORTANT: This is a back-compatibility feature that has
+        # been added in Barman 2.0. It highlights a deprecated
+        # behaviour, and helps users during this transition phase.
+        # It forces 'archiver=on' when both archiver and streaming_archiver
+        # are set to 'off' (default values) and displays a warning,
+        # requesting users to explicitly set the value in the configuration.
+        # When this back-compatibility feature will be removed from Barman
+        # (in a couple of major releases), developers will need to remove
+        # this block completely and reinstate the block of code you find
+        # a few lines below (search for ARCHIVER_OFF_BACKCOMPATIBILITY
+        # throughout the code).
+        if self.config.archiver is False and \
+                self.config.streaming_archiver is False:
+            output.warning("No archiver enabled for server '%s'. "
+                           "Please turn on 'archiver', 'streaming_archiver' "
+                           "or both", self.config.name)
+            output.warning("Forcing 'archiver = on'")
+            self.config.archiver = True
+        # ARCHIVER_OFF_BACKCOMPATIBILITY - END OF CODE
+
+        # Initialize the FileWalArchiver
+        # WARNING: Order of items in self.archivers list is important!
         # The files will be archived in that order.
         if self.config.archiver:
             try:
@@ -187,35 +223,58 @@ class Server(RemoteStatusMixin):
                 self.config.disabled = True
                 self.config.msg_list.append('Unable to initialise the '
                                             'file based archiver')
-        else:
-            # Currently a server MUST have archiver set to on,
-            # otherwise disable the server.
-            self.config.disabled = True
-            self.config.msg_list.append("The option archiver = off "
-                                        "is not yet supported")
 
-        if self.config.streaming_archiver:
+        # Initialize the streaming PostgreSQL connection only when
+        # backup_method is postgres or the streaming_archiver is in use
+        if (self.config.backup_method == 'postgres' or
+                self.config.streaming_archiver):
             try:
                 self.streaming = StreamingConnection(config)
-                self.archivers.append(StreamingWalArchiver(
-                    self.backup_manager))
-            # If the StreamingConnection creation fails, disable the Server
+            # If the StreamingConnection creation fails, disable the server
             except ConninfoException as e:
                 self.config.disabled = True
-                self.config.msg_list.append("streaming_conninfo: " +
+                self.config.msg_list.append("Streaming connection: " +
                                             str(e).strip())
+
+        # Initialize the StreamingWalArchiver
+        # WARNING: Order of items in self.archivers list is important!
+        # The files will be archived in that order.
+        if self.config.streaming_archiver:
+            try:
+                self.archivers.append(StreamingWalArchiver(
+                    self.backup_manager))
+            # If the StreamingWalArchiver creation fails,
+            # disable the server
             except AttributeError as e:
                 _logger.debug(e)
                 self.config.disabled = True
                 self.config.msg_list.append('Unable to initialise the '
                                             'streaming archiver')
-        if len(self.archivers) < 1:
+
+        # IMPORTANT: The following lines of code have been
+        # temporarily commented in order to make the code
+        # back-compatible after the introduction of 'archiver=off'
+        # as default value in Barman 2.0.
+        # When the back compatibility feature for archiver will be
+        # removed, the following lines need to be decommented.
+        # ARCHIVER_OFF_BACKCOMPATIBILITY - START OF CODE
+        # # At least one of the available archive modes should be enabled
+        # if len(self.archivers) < 1:
+        #     self.config.disabled = True
+        #     self.config.msg_list.append(
+        #         "No archiver enabled for server '%s'. "
+        #         "Please turn on 'archiver', 'streaming_archiver' or both"
+        #         % config.name)
+        # ARCHIVER_OFF_BACKCOMPATIBILITY - END OF CODE
+
+        # Sanity check: if file based archiver is disabled, and only
+        # WAL streaming is enabled, a replication slot name must be configured.
+        if not self.config.archiver and self.config.streaming_archiver and \
+                self.config.slot_name is None:
             self.config.disabled = True
             self.config.msg_list.append(
-                "Missing archiver for server %s. "
-                "Enable at least the 'archiver' option in "
-                "the server configuration"
-                % config.name)
+                "Streaming-only archiver requires 'streaming_conninfo' and "
+                "'slot_name' options to be properly configured")
 
         # Set bandwidth_limit
         if self.config.bandwidth_limit:
@@ -335,32 +394,42 @@ class Server(RemoteStatusMixin):
         connections work properly. It checks also that backup directories exist
         (and if not, it creates them).
 
+        The check command will time out after a time interval defined by the
+        check_timeout configuration value (default 30 seconds)
+
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
-        # Check WAL archive
-        self.check_archive(check_strategy)
-        # Check postgres configuration
-        self.check_postgres(check_strategy)
-        # Check barman directories from barman configuration
-        self.check_directories(check_strategy)
-        # Check retention policies
-        self.check_retention_policy_settings(check_strategy)
-        # Check for backup validity
-        self.check_backup_validity(check_strategy)
-        # Executes the backup manager set of checks
-        self.backup_manager.check(check_strategy)
-        # Check if the msg_list of the server
-        # contains messages and output eventual failures
-        self.check_configuration(check_strategy)
+        try:
+            with timeout(self.config.check_timeout):
+                # Check WAL archive
+                self.check_archive(check_strategy)
+                # Check postgres configuration
+                self.check_postgres(check_strategy)
+                # Check barman directories from barman configuration
+                self.check_directories(check_strategy)
+                # Check retention policies
+                self.check_retention_policy_settings(check_strategy)
+                # Check for backup validity
+                self.check_backup_validity(check_strategy)
+                # Executes the backup manager set of checks
+                self.backup_manager.check(check_strategy)
+                # Check if the msg_list of the server
+                # contains messages and output eventual failures
+                self.check_configuration(check_strategy)
 
-        # Executes check() for every archiver, passing
-        # remote status information for efficiency
-        for archiver in self.archivers:
-            archiver.check(check_strategy)
+                # Executes check() for every archiver, passing
+                # remote status information for efficiency
+                for archiver in self.archivers:
+                    archiver.check(check_strategy)
 
-        # Check archiver errors
-        self.check_archiver_errors(check_strategy)
+                # Check archiver errors
+                self.check_archiver_errors(check_strategy)
+        except TimeoutError:
+            # The check timed out.
+            # Add a failed entry to the check strategy for this.
+            check_strategy.result(self.config.name, 'check timeout', False,
+                                  'barman check command timed out')
 
     def check_archive(self, check_strategy):
         """
@@ -369,13 +438,21 @@ class Server(RemoteStatusMixin):
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
-        # Check WAL archiving has been setup
+
+        # Make sure that WAL archiving has been setup
+        # XLOG_DB needs to exist and its size must be > 0
+        # NOTE: we do not need to acquire a lock in this phase
+        xlogdb_empty = True
+        if os.path.exists(self.xlogdb_file_name):
+            with open(self.xlogdb_file_name, "rb") as fxlogdb:
+                if os.fstat(fxlogdb.fileno()).st_size > 0:
+                    xlogdb_empty = False
+
         # NOTE: This check needs to be only visible if it fails
-        with self.xlogdb() as fxlogdb:
-            if os.fstat(fxlogdb.fileno()).st_size == 0:
-                check_strategy.result(
-                    self.config.name, 'WAL archive', False,
-                    'please make sure WAL shipping is setup')
+        if xlogdb_empty:
+            check_strategy.result(
+                self.config.name, 'WAL archive', False,
+                'please make sure WAL shipping is setup')
 
     def check_postgres(self, check_strategy):
         """
@@ -407,7 +484,7 @@ class Server(RemoteStatusMixin):
             # If a streaming connection is available,
             # add its status to the output of the check
             if remote_status['streaming_supported'] is None:
-                hint = 'Streaming connection error'
+                hint = remote_status['connection_error']
             elif not remote_status['streaming_supported']:
                 hint = ('Streaming connection not supported'
                         ' for PostgreSQL < 9.2')
@@ -424,6 +501,74 @@ class Server(RemoteStatusMixin):
                     self.config.name, 'wal_level', False,
                     "please set it to a higher level than 'minimal'")
 
+        # Check the presence and the status of the configured replication slot
+        # This check will be skipped if `slot_name` is undefined
+        if self.config.slot_name:
+            slot = remote_status['replication_slot']
+            # The streaming_archiver is enabled
+            if self.config.streaming_archiver is True:
+                # Error if PostgreSQL is too old
+                if not remote_status['replication_slot_support']:
+                    check_strategy.result(
+                        self.config.name,
+                        'replication slot',
+                        False,
+                        "slot_name parameter set but "
+                        "PostgreSQL server is too old (%s < 9.4)" % (
+                            remote_status['server_txt_version']))
+                # Replication slots are supported
+                else:
+                    # The slot is not present
+                    if slot is None:
+                        check_strategy.result(
+                            self.config.name, 'replication slot', False,
+                            "replication slot '%s' doesn't exist. "
+                            "Please execute "
+                            "'barman receive-wal --create-slot %s'" %
+                            (self.config.slot_name, self.config.name))
+                    else:
+                        # The slot is present but not initialised
+                        if slot.restart_lsn is None:
+                            check_strategy.result(
+                                self.config.name, 'replication slot', False,
+                                "slot '%s' not initialised: "
+                                "is 'receive-wal' running?" %
+                                self.config.slot_name)
+                        # The slot is present but not active
+                        elif slot.active is False:
+                            check_strategy.result(
+                                self.config.name, 'replication slot', False,
+                                "slot '%s' not active: "
+                                "is 'receive-wal' running?" %
+                                self.config.slot_name)
+                        else:
+                            check_strategy.result(self.config.name,
+                                                  'replication slot',
+                                                  True)
+            else:
+                # If the streaming_archiver is disabled and the slot_name
+                # option is present in the configuration, we check that
+                # a replication slot with the specified name is NOT present
+                # and NOT active.
+                # NOTE: This is not a failure, just a warning.
+                if slot is not None:
+                    if slot.restart_lsn \
+                            is not None:
+                        slot_status = 'initialised'
+
+                        # Check if the slot is also active
+                        if slot.active:
+                            slot_status = 'active'
+
+                        # Warn the user
+                        check_strategy.result(
+                            self.config.name,
+                            'replication slot',
+                            True,
+                            "WARNING: slot '%s' is %s but not required "
+                            "by the current config" % (
+                                self.config.slot_name, slot_status))
+
     def _make_directories(self):
         """
         Make backup directories in case they do not exist
@@ -459,7 +604,7 @@ class Server(RemoteStatusMixin):
         :param CheckStrategy check_strategy: the strategy for the management
              of the results of the various checks
         """
-        if self.config.disabled:
+        if len(self.config.msg_list):
             check_strategy.result(self.config.name, 'configuration', False)
             for conflict_paths in self.config.msg_list:
                 output.info("\t\t%s" % conflict_paths)
@@ -558,41 +703,18 @@ class Server(RemoteStatusMixin):
                           "data_directory",
                           "PostgreSQL Data directory",
                           remote_status['data_directory'])
-        output.result('status', self.config.name,
-                      "archive_command",
-                      "PostgreSQL 'archive_command' setting",
-                      remote_status['archive_command'] or
-                      "FAILED (please set it accordingly to documentation)")
-        last_wal = remote_status.get('last_archived_wal')
-        # If PostgreSQL is >= 9.4 we have the last_archived_time
-        if last_wal and remote_status.get('last_archived_time'):
-                last_wal += ", at %s" % (
-                    remote_status['last_archived_time'].ctime())
-        output.result('status', self.config.name,
-                      "last_archived_wal",
-                      "Last archived WAL",
-                      last_wal or "No WAL segment shipped yet")
         if remote_status['current_xlog']:
             output.result('status', self.config.name,
                           "current_xlog",
                           "Current WAL segment",
                           remote_status['current_xlog'])
-        # Set output for WAL archive failures (PostgreSQL >= 9.4)
-        if remote_status.get('failed_count') is not None:
-            remote_fail = str(remote_status['failed_count'])
-            if int(remote_status['failed_count']) > 0:
-                remote_fail += " (%s at %s)" % (
-                    remote_status['last_failed_wal'],
-                    remote_status['last_failed_time'].ctime())
-            output.result('status', self.config.name, 'failed_count',
-                          'Failures of WAL archiver', remote_fail)
-        # Add hourly archive rate if available (PostgreSQL >= 9.4) and > 0
-        if remote_status.get('current_archived_wals_per_second'):
-            output.result(
-                'status', self.config.name,
-                'server_archived_wals_per_hour',
-                'Server WAL archiving rate', '%0.2f/hour' % (
-                    3600 * remote_status['current_archived_wals_per_second']))
+
+    def status_wal_archiver(self):
+        """
+        Status of WAL archiver(s)
+        """
+        for archiver in self.archivers:
+            archiver.status()
 
     def status_retention_policies(self):
         """
@@ -628,6 +750,7 @@ class Server(RemoteStatusMixin):
                       "disabled",
                       "Disabled", self.config.disabled)
         self.status_postgres()
+        self.status_wal_archiver()
         self.status_retention_policies()
         # Executes the backup manager status info method
         self.backup_manager.status()
@@ -660,10 +783,7 @@ class Server(RemoteStatusMixin):
         Shows the server configuration
         """
         # Populate result map with all the required keys
-        result = dict([
-            (key, getattr(self.config, key))
-            for key in self.config.KEYS
-        ])
+        result = self.config.to_json()
         remote_status = self.get_remote_status()
         result.update(remote_status)
         # Backup maximum age section
@@ -820,7 +940,7 @@ class Server(RemoteStatusMixin):
                     wal_info = self.get_wal_info(backup)
                     backup_size += wal_info['wal_size']
                     wal_size = wal_info['wal_until_next_size']
-                except xlog.BadXlogSegmentName as e:
+                except BadXlogSegmentName as e:
                     output.error(
                         "invalid xlog segment name %r\n"
                         "HINT: Please run \"barman rebuild-xlogdb %s\" "
@@ -1292,6 +1412,109 @@ class Server(RemoteStatusMixin):
                         "on server %s. Skipping to the next server"
                         % self.config.name)
 
+    def create_physical_repslot(self):
+        """
+        Create a physical replication slot using the streaming connection
+        """
+        if not self.streaming:
+            output.error("Unable to create a physical replication slot: "
+                         "streaming connection not configured")
+            return
+
+        # Replication slots are not supported by PostgreSQL < 9.4
+        try:
+            if self.streaming.server_version < 90400:
+                output.error("Unable to create a physical replication slot: "
+                             "not supported by '%s' "
+                             "(9.4 or higher is required)" %
+                             self.streaming.server_major_version)
+                return
+        except PostgresException as exc:
+            msg = "Cannot connect to server '%s'" % self.config.name
+            output.error(msg, log=False)
+            _logger.error("%s: %s", msg, str(exc).strip())
+            return
+
+        if not self.config.slot_name:
+            output.error("Unable to create a physical replication slot: "
+                         "slot_name configuration option required")
+            return
+
+        output.info(
+            "Creating physical replication slot '%s' on server '%s'",
+            self.config.slot_name,
+            self.config.name)
+
+        try:
+            self.streaming.create_physical_repslot(self.config.slot_name)
+            output.info("Replication slot '%s' created", self.config.slot_name)
+        except PostgresDuplicateReplicationSlot:
+            output.error("Replication slot '%s' already exists",
+                         self.config.slot_name)
+        except PostgresReplicationSlotsFull:
+            output.error("All replication slots for server '%s' are in use\n"
+                         "Free one or increase the max_replication_slots "
+                         "value on your PostgreSQL server.",
+                         self.config.name)
+        except PostgresException as exc:
+            output.error(
+                "Cannot create replication slot '%s' on server '%s': %s",
+                self.config.slot_name,
+                self.config.name,
+                str(exc).strip())
+
+    def drop_repslot(self):
+        """
+        Drop a replication slot using the streaming connection
+        """
+        if not self.streaming:
+            output.error("Unable to drop a physical replication slot: "
+                         "streaming connection not configured")
+            return
+
+        # Replication slots are not supported by PostgreSQL < 9.4
+        try:
+            if self.streaming.server_version < 90400:
+                output.error("Unable to drop a physical replication slot: "
+                             "not supported by '%s' (9.4 or higher is "
+                             "required)" %
+                             self.streaming.server_major_version)
+                return
+        except PostgresException as exc:
+            msg = "Cannot connect to server '%s'" % self.config.name
+            output.error(msg, log=False)
+            _logger.error("%s: %s", msg, str(exc).strip())
+            return
+
+        if not self.config.slot_name:
+            output.error("Unable to drop a physical replication slot: "
+                         "slot_name configuration option required")
+            return
+
+        output.info(
+            "Dropping physical replication slot '%s' on server '%s'",
+            self.config.slot_name,
+            self.config.name)
+
+        try:
+            self.streaming.drop_repslot(self.config.slot_name)
+            output.info("Replication slot '%s' dropped", self.config.slot_name)
+        except PostgresInvalidReplicationSlot:
+            output.error("Replication slot '%s' does not exist",
+                         self.config.slot_name)
+        except PostgresReplicationSlotInUse as exc:
+            output.error(
+                "Cannot drop replication slot '%s' on server '%s' "
+                "because it is in use.",
+                self.config.slot_name,
+                self.config.name)
+        except PostgresException as exc:
+            output.error(
+                "Cannot drop replication slot '%s' on server '%s': %s",
+                self.config.slot_name,
+                self.config.name,
+                str(exc).strip())
+
     def receive_wal(self, reset=False):
         """
         Enable the reception of WAL files using streaming protocol.
@@ -1324,8 +1547,8 @@ class Server(RemoteStatusMixin):
                     for archiver in self.archivers:
                         archiver.receive_wal(reset)
                 except ArchiverFailure as e:
-                    output.error("Impossible to start a receive-wal process "
-                                 "for server %s: %s" % (self.config.name, e))
+                    output.error(e)
+
         except LockFileBusy:
             # If another process is running for this server,
             if reset:
@@ -1336,6 +1559,14 @@ class Server(RemoteStatusMixin):
                 output.info("Another receive-wal process is already running "
                             "for server %s." % self.config.name)
 
+    @property
+    def xlogdb_file_name(self):
+        """
+        The name of the file containing the XLOG_DB
+        :return str: the name of the file that contains the XLOG_DB
+        """
+        return os.path.join(self.config.wals_directory, self.XLOG_DB)
+
     @contextmanager
     def xlogdb(self, mode='r'):
         """
@@ -1355,7 +1586,7 @@ class Server(RemoteStatusMixin):
         """
         if not os.path.exists(self.config.wals_directory):
             os.makedirs(self.config.wals_directory)
-        xlogdb = os.path.join(self.config.wals_directory, self.XLOG_DB)
+        xlogdb = self.xlogdb_file_name
 
         with ServerXLOGDBLock(self.config.barman_lock_directory,
                               self.config.name):
@@ -1435,6 +1666,15 @@ class Server(RemoteStatusMixin):
                     policy.backup_status(backup_info.backup_id)
             else:
                 backup_ext_info['retention_policy_status'] = None
+
+            # Check any child timeline exists
+            children_timelines = self.get_children_timelines(
+                backup_ext_info['timeline'],
+                forked_after=backup_info.end_xlog)
+
+            backup_ext_info['children_timelines'] = \
+                children_timelines
+
         return backup_ext_info
 
     def show_backup(self, backup_info):
@@ -1446,7 +1686,7 @@ class Server(RemoteStatusMixin):
         try:
             backup_ext_info = self.get_backup_ext_info(backup_info)
             output.result('show_backup', backup_ext_info)
-        except xlog.BadXlogSegmentName as e:
+        except BadXlogSegmentName as e:
             output.error(
                 "invalid xlog segment name %r\n"
                 "HINT: Please run \"barman rebuild-xlogdb %s\" "
@@ -1553,3 +1793,46 @@ class Server(RemoteStatusMixin):
             output.info("  Requires PostgreSQL %s or higher", e)
         except PostgresSuperuserRequired:
             output.info("  Requires superuser rights")
+
+    def get_children_timelines(self, tli, forked_after=None):
+        """
+        Get a list of the children of the passed timeline
+
+        :param int tli: Id of the timeline to check
+        :param str forked_after: XLog location after which the timeline
+          must have been created
+        :return List[xlog.HistoryFileData]: the list of timelines that
+          have the timeline with id 'tli' as parent
+        """
+        if forked_after:
+            forked_after = xlog.parse_lsn(forked_after)
+
+        children = []
+        # Search all the history files after the passed timeline
+        children_tli = tli
+        while True:
+            children_tli += 1
+            history_path = os.path.join(self.config.wals_directory,
+                                        "%08X.history" % children_tli)
+            # If the file doesn't exists, stop searching
+            if not os.path.exists(history_path):
+                break
+
+            # Get content of the file
+            history_info = xlog.decode_history_file(history_path)
+
+            # Save the history only if is reachable from this timeline.
+            for tinfo in history_info:
+                # The history file contains the full genealogy
+                # but we keep only the line with `tli` timeline as parent.
+                if tinfo.parent_tli != tli:
+                    continue
+
+                # We need to return this history info only if this timeline
+                # has been forked after the passed LSN
+                if forked_after and tinfo.switchpoint < forked_after:
+                    continue
+
+                children.append(tinfo)
+
+        return children
