@@ -28,6 +28,12 @@ import logging
 import logging.handlers
 import os
 import pwd
+import signal
+from contextlib import contextmanager
+
+from distutils.version import Version
+
+from barman.exceptions import TimeoutError
 
 _logger = logging.getLogger(__name__)
 
@@ -242,6 +248,9 @@ class BarmanEncoder(json.JSONEncoder):
         # an unicode string
         if hasattr(obj, 'decode') and callable(obj.decode):
             return obj.decode('utf-8', 'replace')
+        # Manage (Loose|Strict)Version objects as strings.
+        if isinstance(obj, Version):
+            return str(obj)
         # Let the base class default method raise the TypeError
         return super(BarmanEncoder, self).default(obj)
 
@@ -292,3 +301,29 @@ def with_metaclass(meta, *bases):
         def __new__(mcs, name, this_bases, d):
             return meta(name, bases, d)
     return type.__new__(Metaclass, 'temporary_class', (), {})
+
+
+@contextmanager
+def timeout(timeout_duration):
+    """
+    ContextManager responsible for timing out the contained
+    block of code after a defined time interval.
+    """
+    # Define the handler for the alarm signal
+    def handler(signum, frame):
+        raise TimeoutError()
+
+    # set the timeout handler
+    previous_handler = signal.signal(signal.SIGALRM, handler)
+    if previous_handler != signal.SIG_DFL:
+        signal.signal(signal.SIGALRM, previous_handler)
+        raise AssertionError("Another timeout is already defined")
+    # set the timeout duration
+    signal.alarm(timeout_duration)
+    try:
+        # Execute the contained block of code
+        yield
+    finally:
+        # Reset the signal
+        signal.alarm(0)
+        signal.signal(signal.SIGALRM, signal.SIG_DFL)
