@@ -20,10 +20,13 @@ This module contains functions to retrieve information about xlog
 files
 """
 
+import collections
 import os
 import re
 
 # xlog file segment name parser (regular expression)
+from barman.exceptions import BadHistoryFileContents, BadXlogSegmentName
+
 _xlog_re = re.compile(r'''
     ^
     ([\dA-Fa-f]{8})                    # everything has a timeline
@@ -39,6 +42,7 @@ _xlog_re = re.compile(r'''
     )
     $
     ''', re.VERBOSE)
+
 # xlog location parser for concurrent backup (regular expression)
 _location_re = re.compile(r'^([\dA-F]+)/([\dA-F]+)$')
 
@@ -47,12 +51,11 @@ XLOG_SEG_SIZE = 1 << 24
 XLOG_SEG_PER_FILE = 0xffffffff // XLOG_SEG_SIZE
 XLOG_FILE_SIZE = XLOG_SEG_SIZE * XLOG_SEG_PER_FILE
 
-
-class BadXlogSegmentName(Exception):
-    """
-    Exception for a bad xlog name
-    """
-    pass
+#: This namedtuple is a container for the information
+#: contained inside history files
+HistoryFileData = collections.namedtuple(
+    'HistoryFileData',
+    'tli parent_tli switchpoint reason')
 
 
 def is_any_xlog_file(path):
@@ -195,7 +198,7 @@ def generate_segment_names(begin, end=None, version=None):
             "Begin segment (%s) and end segment (%s) "
             "must have the same timeline part" % (begin, end))
 
-    # If version is less than 9.3 the last segmen must be skipped
+    # If version is less than 9.3 the last segment must be skipped
     skip_last_segment = version is not None and version < 90300
 
     # Start from the first xlog and generate the segments sequentially
@@ -230,16 +233,137 @@ def hash_dir(path):
         return ''
 
 
-def get_offset_from_location(location):
+def parse_lsn(lsn_string):
     """
-    Calculate a xlog segment offset starting from a xlog location.
+    Transform a string XLOG location, formatted as %X/%X, in the corresponding
+    numeric representation
 
-    :param location: a complete xlog location
-    :return int: a xlog segment offset
+    :param str lsn_string: the string XLOG location, i.e. '2/82000168'
+    :rtype: int
     """
-    match = _location_re.match(location)
-    if match:
-        xlo = int(match.group(2), 16)
-        return xlo % XLOG_SEG_SIZE
-    else:
+    lsn_list = lsn_string.split('/')
+    if len(lsn_list) != 2:
+        raise ValueError('Invalid LSN: %s', lsn_string)
+
+    return (int(lsn_list[0], 16) << 32) + int(lsn_list[1], 16)
+
+
+def diff_lsn(lsn_string1, lsn_string2):
+    """
+    Calculate the difference in bytes between two string XLOG location,
+    formatted as %X/%X
+
+    Tis function is a Python implementation of
+    the ``pg_xlog_location_diff(str, str)`` PostgreSQL function.
+
+    :param str lsn_string1: the string XLOG location, i.e. '2/82000168'
+    :param str lsn_string2: the string XLOG location, i.e. '2/82000168'
+    :rtype: int
+    """
+
+    # If one the input is None returns None
+    if lsn_string1 is None or lsn_string2 is None:
         return None
+    return parse_lsn(lsn_string1) - parse_lsn(lsn_string2)
+
+
+def format_lsn(lsn):
+    """
+    Transform a numeric XLOG location, in the corresponding %X/%X string
+    representation
+
+    :param int lsn: numeric XLOG location
+    :rtype: str
+    """
+    return "%X/%X" % (lsn >> 32, lsn & 0xFFFFFFFF)
+
+
+def location_to_xlogfile_name_offset(location, timeline):
+    """
+    Convert transaction log location string to file_name and file_offset
+
+    This is a reimplementation of pg_xlogfile_name_offset PostgreSQL function
+
+    This method returns a dictionary containing the following data:
+
+         * file_name
+         * file_offset
+
+    :param str location: XLOG location
+    :param int timeline: timeline
+    :rtype: dict
+    """
+    lsn = parse_lsn(location)
+    log = lsn >> 32
+    seg = (lsn & XLOG_FILE_SIZE) >> 24
+    offset = lsn & 0xFFFFFF
+    return {
+        'file_name': encode_segment_name(timeline, log, seg),
+        'file_offset': offset,
+    }
+
+
+def location_from_xlogfile_name_offset(file_name, file_offset):
+    """
+    Convert file_name and file_offset to a transaction log location.
+
+    This is the inverted function of PostgreSQL's pg_xlogfile_name_offset
+    function.
+
+    :param str file_name: a WAL file name
+    :param int file_offset: a numeric offset
+    :rtype: str
+    """
+    decoded_segment = decode_segment_name(file_name)
+    location = ((decoded_segment[1] << 32) +
+                (decoded_segment[2] << 24) +
+                file_offset)
+    return format_lsn(location)
+
+
+def decode_history_file(path):
+    """
+    Read an history file and parse its contents.
+
+    Each line in the file represents a timeline switch, each field is
+    separated by tab, empty lines are ignored and lines starting with '#'
+    are comments.
+
+    Each line is composed by three fields: parentTLI, switchpoint and reason.
+    "parentTLI" is the ID of the parent timeline.
+    "switchpoint" is the WAL position where the switch happened
+    "reason" is an human-readable explanation of why the timeline was changed
+
+    :param path: history file location
+    :return List[HistoryFileData]: information from the history file
+    """
+    lines = []
+    with open(path) as fp:
+        for line in fp:
+            line = line.strip()
+            # Skip comments and empty lines
+            if line.startswith("#"):
+                continue
+            # Skip comments and empty lines
+            if len(line) == 0:
+                continue
+            # Use tab as separator
+            contents = line.split('\t')
+            if len(contents) != 3:
+                # Invalid content of the line
+                raise BadHistoryFileContents(path)
+
+            tli, _, _ = decode_segment_name(path)
+
+            history = HistoryFileData(
+                tli=tli,
+                parent_tli=int(contents[0]),
+                switchpoint=parse_lsn(contents[1]),
+                reason=contents[2])
+            lines.append(history)
+
+    # Empty history file or containing invalid content
+    if len(lines) == 0:
+        raise BadHistoryFileContents(path)
+    else:
+        return lines
